from zope.interface import implements, Interface
from zope.component import adapts

from pcommerce.core import PCommerceMessageFactory as _
from pcommerce.core.interfaces import IPaymentMethod
from pcommerce.payment.invoice.interfaces import IInvoicePayment

FAILED = 1
SUCCESS = 2

class InvoicePayment(object):
    implements(IPaymentMethod, IInvoicePayment)
    adapts(Interface)
    
    title = _('Invoice')
    description = _('Payment on invoice')
    icon = '++resource++pcommerce_payment_invoice_icon.gif'
    logo = '++resource++pcommerce_payment_invoice_logo.gif'
    
    def __init__(self, context):
        self.context = context
        
    def verifyPayment(self, order):
        return getattr(order.paymentdata, 'state', FAILED) is SUCCESS
    
    def mailInfo(self, order, lang=None, customer=False):
        data = order.paymentdata
        address = data.as_customer and order.address or data.address
        if customer:
            return _('invoice_mailinfo_customer', default="""Billing address
${address}""", mapping=dict(address=address.mailInfo(self.context.REQUEST, lang, customer)))
        else:
            return _('invoice_mailinfo', default="""Billing address
${address}

To process the invoice payment click the following link
${link}

To cancel the invoice payment process and fail the order click the following link
${link_failed}""", mapping=dict(address=address.mailInfo(self.context.REQUEST, lang, customer),
                                link='%s/processInvoice?orderid=%s' % (self.context.absolute_url(), order.orderid),
                                link_failed='%s/processInvoice?orderid=%s&failed=1' % (self.context.absolute_url(), order.orderid)))



from pcommerce.core.data import PaymentData

def InvoicePaymentData(as_customer=True, address=None):
    data = PaymentData('pcommerce.payment.invoice')
    data.as_customer = as_customer
    data.address = address
    return data



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from pcommerce.core.interfaces import IAddressFactory
from pcommerce.payment.invoice.browser.base import InvoiceBase
from pcommerce.payment.invoice.data import InvoicePaymentData

class InvoicePayment(InvoiceBase):
    template = ViewPageTemplateFile('payment.pt')
    
    def validate(self):
        self.errors = {}
        as_customer = self.request.form.get('invoice_address_as_customer', False)
        if not as_customer:
            factory = IAddressFactory(self.request)
            self.errors = factory.validate('invoice_address')
        return len(self.errors) == 0
    
    def process(self):
        self.data = InvoicePaymentData()
        as_customer = self.request.form.get('invoice_address_as_customer', False)
        if not as_customer:
            factory = IAddressFactory(self.request)
            self.data.address = factory.create('invoice_address')
        self.data.as_customer = as_customer
        props = getToolByName(self.context, 'portal_properties').pcommerce_properties
        self.data.pretaxcharge = props.getProperty('invoice_pretaxcharge', 0.0)
        self.data.posttaxcharge = props.getProperty('invoice_posttaxcharge', 0.0)
        return self.data



from zope.interface import implements

from Products.Five.browser import BrowserView

from pcommerce.core.interfaces import IPaymentView, IOrder, IShoppingCart

from pcommerce.payment.invoice.data import InvoicePaymentData

class InvoiceBase(BrowserView):
    implements(IPaymentView)
    
    errors = {}
    
    def __init__(self, payment, request):
        self.payment = payment
        self.context = payment.context
        self.request = request
        self.cart = IShoppingCart(self.context)
        self.order = IOrder(self.context)
        self.data = self.order.paymentdata or InvoicePaymentData()
    
    def __call__(self):
        return self.template()
    
    def validate(self):
        return True
    
    def renders(self):
        return True
        
    def process(self):
        return
    
    @property
    def address(self):
        if self.data.as_customer:
            return self.order.address
        return self.data.address
    
    @property
    def address_as_customer(self):
        if self.request.get('invoice_hidden', False):
            return self.request.get('invoice_address_as_customer', False)
        return self.data.as_customer



from xml.dom import minidom

from Products.Five.browser import BrowserView

from pcommerce.core.interfaces import IPaymentProcessor, IOrderRegistry
from pcommerce.payment.invoice.plugin import FAILED, SUCCESS

class ProcessInvoice(BrowserView):
    """process Invoice payments
    """

    def __call__(self, orderid, failed=False):
        orderid = int(orderid)
        registry = IOrderRegistry(self.context)
        order = registry.getOrder(orderid)
        if order is not None and order.paymentid == 'pcommerce.payment.invoice':
            order.paymentdata.state = failed and FAILED or SUCCESS
        processor = IPaymentProcessor(self.context)
        return processor.processOrder(orderid, 'pcommerce.payment.invoice')



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from pcommerce.payment.invoice.browser.base import InvoiceBase

class InvoiceConfirmation(InvoiceBase):
    template = ViewPageTemplateFile('confirmation.pt')